/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_OMP_PARALLEL_EFFICIENCY_TEST_H
#define CUBELIB_BSPOP_HYBRID_OMP_PARALLEL_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "BSPOPHybridMPIParallelEfficiencyTest.h"
#include "BSPOPHybridParallelEfficiencyTest.h"

namespace bscanalysis
{
class BSPOPHybridOMPParallelEfficiencyTest : public popcalculation::PerformanceTest
{
private:

    BSPOPHybridParallelEfficiencyTest*    par_eff;
    BSPOPHybridMPIParallelEfficiencyTest* mpi_eff;

    void
    calculate();

    void
    calculate( const cube::list_of_cnodes& );

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "OpenMP Parallel Efficiency (OMPE) reveals the inefficiency in OpenMP threads.\n"
            "OMPE can be computed directly or as a division Hybrid Parallel Efficiency by MPI Parallel Efficiency :\n"
            "\tOMPE = avg(computation time) / avg(time outside of MPI)\n"
            "\t     = Hybrid Parallel Efficiency / MPI Parallel Efficiency";
        std::string not_active_text =
            "OpenMP Parallel Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    BSPOPHybridOMPParallelEfficiencyTest(  cube::CubeProxy*,
                                           BSPOPHybridParallelEfficiencyTest*,
                                           BSPOPHybridMPIParallelEfficiencyTest* );

    virtual
    ~BSPOPHybridOMPParallelEfficiencyTest()
    {
    };
// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;


    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( par_eff != nullptr && mpi_eff != nullptr )
        {
            prereqs.push_back( par_eff );
            prereqs.push_back( mpi_eff );
        }
        return prereqs;
    }
};
};
#endif // ADVISER_RATING_WIDGET_H
