/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2023-2024                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_GPU_COMMUNICATION_EFFICENCY_TEST_H
#define CUBELIB_POP_GPU_COMMUNICATION_EFFICENCY_TEST_H

#include <string>
#include "PerformanceTest.h"

namespace popcalculation
{
class POPGPUParallelEfficiencyTest;

class POPGPUCommunicationEfficiencyTest : public popcalculation::PerformanceTest
{
    friend class POPGPUParallelEfficiencyTest;

private:
    cube::Metric* max_cuda_kernel_executions;
    cube::Metric* max_runtime;

    cube::list_of_metrics lmax_cuda_kernel_executions;

protected:

    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "GPU Communication Efficiency (GPUCommE) is the maximum across all GPU streams of the ratio \n"
            "between useful computation time and total run-time:\n"

            "\tGPUCommE=maximum across gpu streams(computation time /wallclock runtime) \n"

            "CommE identifies when code is inefficient because it spends a large amount of time \n"
            " in data transfer rather than performing useful computations. \n\n";
        std::string not_active_text =
            "GPU Communication Efficiency is only avialable for the application, which is actually using the GPU. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca or has no GPU-related locations and metrics,"
            "the GPU Communication Efficiency cannot be calculated.\n";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    POPGPUCommunicationEfficiencyTest(
        cube::CubeProxy* );

    virtual
    ~POPGPUCommunicationEfficiencyTest()
    {
    };

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );


    virtual
    double
    analyze( const cube::list_of_cnodes& cnodes,
             cube::LocationGroup*        _lg = nullptr ) const;

// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // ADVISER_RATING_WIDGET_H
