/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_HYBRID_PARALLEL_EFFICIENCY_TEST_ADD_H
#define CUBELIB_POP_HYBRID_PARALLEL_EFFICIENCY_TEST_ADD_H

#include <string>
#include "PerformanceTest.h"
#include "POPHybridProcessEfficiencyTestAdd.h"
#include "POPHybridThreadEfficiencyTestAdd.h"

namespace hybaddanalysis
{
class POPHybridParallelEfficiencyTestAdd : public popcalculation::PerformanceTest
{
private:
    cube::Metric* pop_avg_comp;
    cube::Metric* max_runtime;

    cube::list_of_metrics max_runtime_metrics;


    POPHybridProcessEfficiencyTestAdd* proc_eff;
    POPHybridThreadEfficiencyTestAdd*  thread_eff;

    void
    calculate();

    void
    calculate( const cube::list_of_cnodes& );


protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Parallel Efficiency (PE) reveals the inefficiency in processes and threads utilization. \n"
            "These are measured with Process Efficiency and Thread Efficiency, \n"
            "and PE can be computed directly or as a sum of these two sub-metrics minus one:\n"
            "\tPE = avg(computation time) / max(runtime) \n"
            "\t   = Process Efficiency + Thread Efficiency - 1 ";
        std::string not_active_text =
            "Parallel Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    POPHybridParallelEfficiencyTestAdd( cube::CubeProxy*,
                                        POPHybridProcessEfficiencyTestAdd* proc_eff,
                                        POPHybridThreadEfficiencyTestAdd*  thread_eff );

    virtual
    ~POPHybridParallelEfficiencyTestAdd()
    {
    };
// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;


    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( proc_eff != nullptr && thread_eff != nullptr )
        {
            prereqs.push_back( proc_eff );
            prereqs.push_back( thread_eff );
        }
        return prereqs;
    }
};
};
#endif // ADVISER_RATING_WIDGET_H
