/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2023                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_MPI_IO_TIME_H
#define CUBELIB_POP_MPI_IO_TIME_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPIOEfficiencyTest;
class POPMpiIOTime : public PerformanceTest
{
    friend class POPIOEfficiencyTest;

private:
    cube::Metric*         execution;
    cube::Metric*         mpi_io;
    cube::list_of_metrics lmpi_io_metrics;

protected:


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "MPI IO Efficiency shows the fraction of execution time spent in MPI IO calls\n"
            "In this analysis MPI IO is computed as :\n"
            "\tMPI IO = time in MPI IO calls / total runtime \n";
        std::string not_active_text =
            "MPI IO Efficiency  metric is available for such Score-P/Scalasca measurement, where MPI IO calls are present. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time and hence IO Time .\n"
            "In this case this metric reported as NAN.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPMpiIOTime( cube::CubeProxy* );

    virtual
    ~POPMpiIOTime()
    {
    };
// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    virtual
    inline
    bool
    isPercent() const
    {
        return false;
    }

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isNormalized() const // returns true, if range goes from 0 to 1. false -> otherwise
    {
        return false;
    }

    inline
    virtual
    std::string
    units() const
    {
        return "sec";
    }
};
};
#endif // POP_ipc_TEST_H
